#include "configuration.hpp"
#include "context.hpp"

/* MGLContext.configure(configuration)
 */
PyObject * MGLContext_meth_configure(MGLContext * self, PyObject * configuration) {
    Py_RETURN_NONE;
}
